%FRONTAL FORCINGS rignot arma class definition
%
%   Usage:
%      frontalforcingsrignotarma=frontalforcingsrignotarma();

classdef frontalforcingsrignotarma
	properties (SetAccess=public) 
		num_basins           = 0;
		num_params           = 0;
		num_breaks           = 0;
		polynomialparams     = NaN;
		datebreaks           = NaN;
		ar_order             = 0;
		ma_order             = 0;
		arma_timestep        = 0;
		arlag_coefs          = NaN;
		malag_coefs          = NaN;
		monthly_effects      = NaN;
		basin_id             = NaN;
		subglacial_discharge = NaN;
	end
	methods
		function self = frontalforcingsrignotarma(varargin) % {{{
			switch nargin
				case 0
					self=setdefaultparameters(self);
				case 1
					inputstruct=varargin{1};
					list1 = properties('frontalforcingsrignotarma');
					list2 = fieldnames(inputstruct);
					for i=1:length(list1)
						fieldname = list1{i};
						if ismember(fieldname,list2),
							self.(fieldname) = inputstruct.(fieldname);
						end
					end
				otherwise
					error('constructor not supported');
			end
		end % }}}
		function self = extrude(self,md) % {{{
		    % nothing for now
		end % }}}
		function self = setdefaultparameters(self) % {{{

			self.basin_id             = NaN;
			self.num_basins           = 0;
			self.subglacial_discharge = NaN;
			self.ar_order             = 0.0; %autoregression model of order 0
			self.ma_order             = 0.0; %moving-average model of order 0

		end % }}}
		function md = checkconsistency(self,md,solution,analyses) % {{{
			%Early return
			if (~strcmp(solution,'TransientSolution') | md.transient.ismovingfront==0), return; end

			nbas = md.frontalforcings.num_basins;
			nprm = md.frontalforcings.num_params;
			nbrk = md.frontalforcings.num_breaks;
			md = checkfield(md,'fieldname','frontalforcings.num_basins','numel',1,'NaN',1,'Inf',1,'>',0);
			md = checkfield(md,'fieldname','frontalforcings.num_params','numel',1,'NaN',1,'Inf',1,'>',0);
			md = checkfield(md,'fieldname','frontalforcings.num_breaks','numel',1,'NaN',1,'Inf',1,'>=',0);
         md = checkfield(md,'fieldname','frontalforcings.basin_id','Inf',1,'>=',0,'<=',md.frontalforcings.num_basins,'size',[md.mesh.numberofelements 1]);
			md = checkfield(md,'fieldname','frontalforcings.subglacial_discharge','>=',0,'NaN',1,'Inf',1,'timeseries',1);
			if(nbas>1 && nbrk>=1 && nprm>1)
			   md = checkfield(md,'fieldname','frontalforcings.polynomialparams','NaN',1,'Inf',1,'size',[nbas,nbrk+1,nprm],'numel',nbas*(nbrk+1)*nprm); 
			elseif(nbas==1)
				md = checkfield(md,'fieldname','frontalforcings.polynomialparams','NaN',1,'Inf',1,'size',[nprm,nbrk+1],'numel',nbas*(nbrk+1)*nprm);
			elseif(nbrk==0)
				md = checkfield(md,'fieldname','frontalforcings.polynomialparams','NaN',1,'Inf',1,'size',[nbas,nprm],'numel',nbas*(nbrk+1)*nprm);
			elseif(nprm==1)
				md = checkfield(md,'fieldname','frontalforcings.polynomialparams','NaN',1,'Inf',1,'size',[nbas,nbrk+1],'numel',nbas*(nbrk+1)*nprm);
			end
			md = checkfield(md,'fieldname','frontalforcings.ar_order','numel',1,'NaN',1,'Inf',1,'>=',0);
         md = checkfield(md,'fieldname','frontalforcings.ma_order','numel',1,'NaN',1,'Inf',1,'>=',0);
         md = checkfield(md,'fieldname','frontalforcings.arma_timestep','numel',1,'NaN',1,'Inf',1,'>=',md.timestepping.time_step); %ARMA time step cannot be finer than ISSM timestep
			md = checkfield(md,'fieldname','frontalforcings.arlag_coefs','NaN',1,'Inf',1,'size',[md.frontalforcings.num_basins,md.frontalforcings.ar_order]);
			md = checkfield(md,'fieldname','frontalforcings.malag_coefs','NaN',1,'Inf',1,'size',[md.frontalforcings.num_basins,md.frontalforcings.ma_order]);
			if(nbrk>0)
				md = checkfield(md,'fieldname','frontalforcings.datebreaks','NaN',1,'Inf',1,'size',[nbas,nbrk]);
			elseif(numel(md.frontalforcings.datebreaks)==0 || all(isnan(md.frontalforcings.datebreaks)))
				;
			else
				error('md.frontalforcings.num_breaks is 0 but md.frontalforcings.datebreaks is not empty');
			end
			if(any(~isnan(md.frontalforcings.monthly_effects)))
				md = checkfield(md,'fieldname','frontalforcings.monthly_effects','NaN',1,'Inf',1,'size',[md.frontalforcings.num_basins,12]);
				if(md.timestepping.time_step>=1)
					error('md.frontalforcings.monthly_effects are provided but md.timestepping.time_step>=1');
				end
			end

		end % }}}
		function disp(self) % {{{
			disp(sprintf('   Frontalforcings parameters:'));
			fielddisplay(self,'num_basins','number of different basins');
         fielddisplay(self,'basin_id','basin number assigned to each element [unitless]');
         fielddisplay(self,'subglacial_discharge','sum of subglacial discharge for each basin [m/d]');
			fielddisplay(self,'num_breaks','number of different breakpoints in the piecewise-polynomial (separating num_breaks+1 periods)');
			fielddisplay(self,'num_params','number of different parameters in the piecewise-polynomial (1:intercept only, 2:with linear trend, 3:with quadratic trend, etc.)');
         fielddisplay(self,'polynomialparams','coefficients for the polynomial (const,trend,quadratic,etc.),dim1 for basins,dim2 for periods,dim3 for orders');
			disp(sprintf('%51s  ex: polyparams=cat(3,intercepts,trendlinearcoefs,trendquadraticcoefs)',' '));
         fielddisplay(self,'datebreaks','dates at which the breakpoints in the piecewise polynomial occur (1 row per basin) [yr]');
         fielddisplay(self,'ar_order','order of the autoregressive model [unitless]');
         fielddisplay(self,'ma_order','order of the moving-average model [unitless]');
         fielddisplay(self,'arma_timestep','time resolution of the autoregressive model [yr]');
         fielddisplay(self,'arlag_coefs','basin-specific vectors of AR lag coefficients [unitless]');
         fielddisplay(self,'malag_coefs','basin-specific vectors of MA lag coefficients [unitless]');
         fielddisplay(self,'monthly_effects','basin-specific monthly values of TF added at corresponding month (default: all 0) [°C]');
		end % }}}
		function marshall(self,prefix,md,fid) % {{{
			yts=md.constants.yts;
			nbas = md.frontalforcings.num_basins;
			nprm = md.frontalforcings.num_params;
			nper = md.frontalforcings.num_breaks+1;
			% Scale the parameters %
			polyparamsScaled   = md.frontalforcings.polynomialparams;
			polyparams2dScaled = zeros(nbas,nper*nprm);
			if(nprm>1)
				% Case 3D %
				if(nbas>1 && nper>1)
					for(ii=[1:nprm])
						polyparamsScaled(:,:,ii) = polyparamsScaled(:,:,ii)*((1/yts)^(ii-1));
					end
					% Fit in 2D array %
					for(ii=[1:nprm])
						jj = 1+(ii-1)*nper;
						polyparams2dScaled(:,jj:jj+nper-1) = polyparamsScaled(:,:,ii);
					end
				% Case 2D and higher-order params at increasing row index %
				elseif(nbas==1)
					for(ii=[1:nprm])
						polyparamsScaled(ii,:) = polyparamsScaled(ii,:)*((1/yts)^(ii-1));
					end
					% Fit in row array %
					for(ii=[1:nprm])
						jj = 1+(ii-1)*nper;
						polyparams2dScaled(1,jj:jj+nper-1) = polyparamsScaled(ii,:);
					end
				% Case 2D and higher-order params at incrasing column index %
				elseif(nper==1)
					for(ii=[1:nprm])
						polyparamsScaled(:,ii) = polyparamsScaled(:,ii)*((1/yts)^(ii-1));
					end
					% 2D array is already in correct format %
					polyparams2dScaled = polyparamsScaled;
				end
			else
				% 2D array is already in correct format and no need for scaling %
				polyparams2dScaled = polyparamsScaled;
			end
			if(any(isnan(md.frontalforcings.monthly_effects))) %monthly effects not provided, set to 0
				meffects = zeros(md.frontalforcings.num_basins,12);
			else
				meffects = md.frontalforcings.monthly_effects;
			end
			if(nper==1) %a single period (no break date)
				dbreaks = zeros(nbas,1); %dummy
			else
				dbreaks = md.frontalforcings.datebreaks;
			end

			WriteData(fid,prefix,'name','md.frontalforcings.parameterization','data',3,'format','Integer');
			WriteData(fid,prefix,'object',self,'class','frontalforcings','fieldname','num_basins','format','Integer');
			WriteData(fid,prefix,'object',self,'class','frontalforcings','fieldname','num_breaks','format','Integer');
			WriteData(fid,prefix,'object',self,'class','frontalforcings','fieldname','num_params','format','Integer');
			WriteData(fid,prefix,'object',self,'class','frontalforcings','fieldname','subglacial_discharge','format','DoubleMat','mattype',1,'timeserieslength',md.mesh.numberofvertices+1,'yts',yts);
         WriteData(fid,prefix,'object',self,'class','frontalforcings','fieldname','ar_order','format','Integer');
         WriteData(fid,prefix,'object',self,'class','frontalforcings','fieldname','ma_order','format','Integer');
         WriteData(fid,prefix,'object',self,'class','frontalforcings','fieldname','arma_timestep','format','Double','scale',yts);
         WriteData(fid,prefix,'object',self,'class','frontalforcings','fieldname','basin_id','data',self.basin_id-1,'name','md.frontalforcings.basin_id','format','IntMat','mattype',2); %0-indexed
         WriteData(fid,prefix,'data',polyparams2dScaled,'name','md.frontalforcings.polynomialparams','format','DoubleMat');
         WriteData(fid,prefix,'object',self,'class','frontalforcings','fieldname','arlag_coefs','format','DoubleMat','name','md.frontalforcings.arlag_coefs','yts',yts);
         WriteData(fid,prefix,'object',self,'class','frontalforcings','fieldname','malag_coefs','format','DoubleMat','name','md.frontalforcings.malag_coefs','yts',yts);
         WriteData(fid,prefix,'data',dbreaks,'name','md.frontalforcings.datebreaks','format','DoubleMat','scale',yts);
         WriteData(fid,prefix,'data',meffects,'name','md.frontalforcings.monthly_effects','format','DoubleMat');
		end % }}}
	end
end
